import { Config } from "../config";
import { Location, Source } from "../context";
import { DOMTree, HtmlElement } from "../dom";
import { ElementReadyEvent, EventCallback, EventHandler } from "../event";
import { AttributeEvent, ConditionalEvent, DirectiveEvent, DoctypeEvent, DOMReadyEvent, Event, TagCloseEvent, TagOpenEvent, WhitespaceEvent } from "../event";
import { Token, TokenStream, TokenType } from "../lexer";
/**
 * Parse HTML document into a DOM tree.
 */
export declare class Parser {
    private readonly event;
    private readonly metaTable;
    private dom;
    /**
     * Create a new parser instance.
     *
     * @param config - Configuration
     */
    constructor(config: Config);
    /**
     * Parse HTML markup.
     *
     * @param source - HTML markup.
     * @returns DOM tree representing the HTML markup.
     */
    parseHtml(source: string | Source): DOMTree;
    /**
     * Detect optional end tag.
     *
     * Some tags have optional end tags (e.g. <ul><li>foo<li>bar</ul> is
     * valid). The parser handles this by checking if the element on top of the
     * stack when is allowed to omit.
     */
    private closeOptional;
    protected consumeTag(source: Source, startToken: Token, tokenStream: TokenStream): void;
    protected closeElement(source: Source, node: HtmlElement, active: HtmlElement, location: Location): void;
    /**
     * Discard tokens until the end tag for the foreign element is found.
     */
    protected discardForeignBody(source: Source, foreignTagName: string, tokenStream: TokenStream): void;
    protected consumeAttribute(source: Source, node: HtmlElement, token: Token, next?: Token): void;
    /**
     * Take attribute value token and return a new location referring to only the
     * value.
     *
     * foo="bar"    foo='bar'    foo=bar    foo      foo=""
     *      ^^^          ^^^         ^^^    (null)   (null)
     */
    private getAttributeValueLocation;
    protected consumeDirective(token: Token): void;
    /**
     * Consumes doctype tokens. Emits doctype event.
     */
    protected consumeDoctype(startToken: Token, tokenStream: TokenStream): void;
    /**
     * Return a list of tokens found until the expected token was found.
     */
    protected consumeUntil(tokenStream: TokenStream, search: TokenType): IterableIterator<Token>;
    private next;
    /**
     * Listen on events.
     *
     * @param event - Event name.
     * @param listener - Event callback.
     * @returns A function to unregister the listener.
     */
    on(event: string, listener: EventCallback): () => void;
    /**
     * Listen on single event. The listener is automatically unregistered once the
     * event has been received.
     *
     * @param event - Event name.
     * @param listener - Event callback.
     * @returns A function to unregister the listener.
     */
    once(event: string, listener: EventCallback): () => void;
    /**
     * Defer execution. Will call function sometime later.
     *
     * @param cb - Callback to execute later.
     */
    defer(cb: () => void): void;
    /**
     * Trigger event.
     *
     * @param {string} event - Event name
     * @param {Event} data - Event data
     */
    protected trigger(event: "tag:open", data: TagOpenEvent): void;
    protected trigger(event: "tag:close", data: TagCloseEvent): void;
    protected trigger(event: "element:ready", data: ElementReadyEvent): void;
    protected trigger(event: "dom:load", data: Event): void;
    protected trigger(event: "dom:ready", data: DOMReadyEvent): void;
    protected trigger(event: "doctype", data: DoctypeEvent): void;
    protected trigger(event: "attr", data: AttributeEvent): void;
    protected trigger(event: "whitespace", data: WhitespaceEvent): void;
    protected trigger(event: "conditional", data: ConditionalEvent): void;
    protected trigger(event: "directive", data: DirectiveEvent): void;
    /**
     * @hidden
     */
    getEventHandler(): EventHandler;
    /**
     * Appends a text node to the current element on the stack.
     */
    private appendText;
    /**
     * Trigger close events for any still open elements.
     */
    private closeTree;
}
